// -*- Java -*-
/*!
 * @file  Human_CheckerImpl.java
 * @brief Human_Checker
 * @date  $Date$
 *
 * $Id$
 */

import connector.Connector;
import jp.go.aist.rtm.RTC.DataFlowComponentBase;
import jp.go.aist.rtm.RTC.Manager;
import jp.go.aist.rtm.RTC.port.InPort;
import jp.go.aist.rtm.RTC.util.DataRef;
import jp.go.aist.rtm.RTC.util.StringHolder;
import RTC.ReturnCode_t;
import RTC.TimedLongSeq;

/*!
 * @class Human_CheckerImpl
 * @brief Human_Checker
 *
 * @author Daiki Nomiyama, Shinichi Ishida, Shogo Namatame, Nobuto Matsuhira / SHIBAURA INSTITUTE OF TECHNOLOGY /  3-7-5 TOYOSU KOUTO-KU TOKYO 135-8548
 *@@@@ Email:md14067@shibaura-it.ac.jp
 *         Tell : 03-5859-8073
 * @author 쌩R,Γc^CcڏˌCyMl@ŉYHƑw@135-8548 s]LF3-7-5
 * 
 * CZXFCBSDCZXKp܂D
 *
 *
 *  Robot Service Network Protocol2.2 Specification
 *
 *
 * $Id$
 */
public class Human_CheckerImpl extends DataFlowComponentBase {

  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
	public Human_CheckerImpl(Manager manager) {  
        super(manager);
        // <rtc-template block="initializer">
        m_CountData_val = new TimedLongSeq();
        m_CountData = new DataRef<TimedLongSeq>(m_CountData_val);
        m_CountDataIn = new InPort<TimedLongSeq>("Count_data", m_CountData);
        // </rtc-template>

    }

    /**
     *
     * The initialize action (on CREATED->ALIVE transition)
     * formaer rtc_init_entry() 
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onInitialize() {
        // Registration: InPort/OutPort/Service
        // <rtc-template block="registration">
        // </rtc-template>
        addInPort("Count_data", m_CountDataIn);
        // </rtc-template>
        bindParameter("robot_id", m_robot_id, "Robo:1");
        bindParameter("password", m_password, "8073");
        bindParameter("end_point", m_end_point, "http://localhost:8080/PocoServer/services");
        return super.onInitialize();
    }

    /***
     *
     * The finalize action (on ALIVE->END transition)
     * formaer rtc_exiting_entry()
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onFinalize() {
//        return super.onFinalize();
//    }

    /***
     *
     * The startup action when ExecutionContext startup
     * former rtc_starting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStartup(int ec_id) {
//        return super.onStartup(ec_id);
//    }

    /***
     *
     * The shutdown action when ExecutionContext stop
     * former rtc_stopping_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onShutdown(int ec_id) {
//        return super.onShutdown(ec_id);
//    }

    /***
     *
     * The activated action (Active state entry action)
     * former rtc_active_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    protected ReturnCode_t onActivated(int ec_id) {
    	Connector connector = Connector.getInstance();
    	connector.connect(m_end_point.value, m_robot_id.value, m_password.value);
        return super.onActivated(ec_id);
    }

    /***
     *
     * The deactivated action (Active state exit action)
     * former rtc_active_exit()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onDeactivated(int ec_id) {
    	Connector.getInstance().disconnect();
        return super.onDeactivated(ec_id);
    }

    /***
     *
     * The execution action that is invoked periodically
     * former rtc_active_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onExecute(int ec_id) {
    	
    	if(m_CountDataIn.isNew()){
    		m_CountDataIn.read();
    		int in = m_CountData.v.data[0];
    		int out = m_CountData.v.data[1];
    		int wait = m_CountData.v.data[2];
    		String datas = "<Poco><id>"+m_robot_id.value+"</id><in>"+String.valueOf(in)+"</in>" +
    				"<out>"+String.valueOf(out)+"</out><wait>"+String.valueOf(wait)+"</wait></Poco>";
    		Connector.getInstance().push_data(datas);
    	}
    	
        return super.onExecute(ec_id);
    }

    /***
     *
     * The aborting action when main logic error occurred.
     * former rtc_aborting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//  @Override
//  public ReturnCode_t onAborting(int ec_id) {
//      return super.onAborting(ec_id);
//  }

    /***
     *
     * The error action in ERROR state
     * former rtc_error_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    public ReturnCode_t onError(int ec_id) {
        return super.onError(ec_id);
    }

    /***
     *
     * The reset action that is invoked resetting
     * This is same but different the former rtc_init_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onReset(int ec_id) {
        return super.onReset(ec_id);
    }

    /***
     *
     * The state update action that is invoked after onExecute() action
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStateUpdate(int ec_id) {
//        return super.onStateUpdate(ec_id);
//    }

    /***
     *
     * The action that is invoked when execution context's rate is changed
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onRateChanged(int ec_id) {
//        return super.onRateChanged(ec_id);
//    }
//
	// Configuration variable declaration
	// <rtc-template block="config_declare">
    /*!
     * 
     * - Name:  robot_id
     * - DefaultValue: poco
     */
    protected StringHolder m_robot_id = new StringHolder();
    /*!
     * 
     * - Name:  password
     * - DefaultValue: 8073
     */
    protected StringHolder m_password = new StringHolder();
    /*!
     * 
     * - Name:  end_point
     * - DefaultValue: http://localhost:8080/PocoServer/services
     */
    protected StringHolder m_end_point = new StringHolder();
	// </rtc-template>

    // DataInPort declaration
    // <rtc-template block="inport_declare">
    protected TimedLongSeq m_CountData_val;
    protected DataRef<TimedLongSeq> m_CountData;
    /*!
     */
    protected InPort<TimedLongSeq> m_CountDataIn;

    
    // </rtc-template>

    // DataOutPort declaration
    // <rtc-template block="outport_declare">
    
    // </rtc-template>

    // CORBA Port declaration
    // <rtc-template block="corbaport_declare">
    
    // </rtc-template>

    // Service declaration
    // <rtc-template block="service_declare">
    
    // </rtc-template>

    // Consumer declaration
    // <rtc-template block="consumer_declare">
    
    // </rtc-template>


}

