package servlet;

import java.io.BufferedOutputStream;
import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.robotservices.v02.exception.RSiException;
import org.robotservices.v02.profile.acceptor.IWorkerTool;
import org.robotservices.v02.profile.common.AttachedFile;
import org.robotservices.v02.profile.common.Detail_info;
import org.robotservices.v02.profile.common.Ret_value;
import org.robotservices.v02.profile.invoker.IMultimedia_profile;
import org.robotservices.v02.profile.invoker.InvokerProfileFactory;

import acceptor.CameraCallback;
import acceptor.CameraSessionManager;
import acceptor.RobotWorkerManager;

import com.fujitsu.rsi.helper.MultimediaProfileHelper;

public class CameraServlet  extends HttpServlet{

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;
	public boolean display_mode = true;
	int interval = -1;
	protected void doPost(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {
		//doGetが呼び出された際にdoPostを呼び出すようにしている。
		req.setCharacterEncoding("Windows-31J");
		//リクエストパラメータ
		String robotid = req.getParameter("robot_id");
		String mode = req.getParameter("mode");
		String interval_str = req.getParameter("interval");
		//URLにアクセスした際はリクエストパラメータには何も入っていない。
		//そこでrobotid==nullの時はcamera.jspを表示する
		if(robotid == null){
			req.getRequestDispatcher("/jsp/camera.jsp").forward(req, res);
			return;
		}
		//画像を配信するか、配信停止するのかの条件
		if(mode != null){
			if(mode.equals("display")){
				display_mode = true;
			}
			else if(mode.equals("stop")){
				display_mode = false;
			}
		}
		//各ロボット毎に配信idを設定しており、これを用いることでdistribute_～系のメソッドを扱うことができる。
		long distribution_id = 0;
		long conv_id = 0;
		//workertoolの取得
		RobotWorkerManager rwm = RobotWorkerManager.getInstance();
		IWorkerTool workertool = rwm.getWorkerTool(robotid);
		//このクラスはのちに取得するMultimedia Profileのインターフェースを保存する。
		CameraSessionManager csm = CameraSessionManager.getInstance();
		//workertoolが必ずしもあるわけではないのでここでチェック
		if(workertool != null){
			conv_id = workertool.getConv_id();
			//callback関数を呼び出す．存在しない場合はnullが返る．初回はnullが返る
			CameraCallback imgCallback = csm.getCallback(robotid);
			IMultimedia_profile mp = null;
			//callbackクラスが存在しない場合は作成する
			//最初のリクエスト（配信を行うリクエスト）ではcallbackクラスのインスタンスが生成されていないのでここでインスタンスを作成する。
			if(imgCallback == null){
				System.out.println("imgCallback == null");
				imgCallback = new CameraCallback();
				//callback関数を登録する
				csm.setCallback(robotid,imgCallback);
				InvokerProfileFactory factory = workertool.getProfileFactory();
				//Multimedia Profileのインターフェースを取得
				mp = factory.getMultimedia_profile();
				//multimedia profileの保存
				csm.setMultimediaProfile(robotid, mp);
				try {
					//配信間隔のデータはStringなのでintに変換
					interval = Integer.valueOf(interval_str);
					//配信間隔は100ms以下にしない.
					if(interval<100){
						interval = 100;
					}
					//distribute_camera_imageの呼び出し
					Ret_value ret = mp.distribute_camera_image(conv_id, "",interval , "", imgCallback);
					Detail_info info = ret.get_detail_info();
					//distribution_idを取得・登録
					distribution_id = info.get_distribution_id();
					csm.setDistributionId(robotid,Long.toString(distribution_id));
				} catch (RSiException e) {
					// TODO 自動生成された catch ブロック
					e.printStackTrace();
				}
			}
			//callback関数が存在する場合は画像を返信する
			//二回目以降
			else if(imgCallback != null){
				//配信モード
				if(display_mode == true){
					try {
						//semaphoreは同時に実行できるスレッドを制御できる。ここではこのcallback用のスレッドを許可している。
						imgCallback.semaphore.acquire();
						//Ret_valueに何も入っていないかチェック
						if(imgCallback.ret_value != null){
							MultimediaProfileHelper helper = new MultimediaProfileHelper(imgCallback.ret_value);
							//複数枚の画像がある場合もある
							int filecount = helper.getFileCount();
							for (int i = 0; i < filecount; i++) {
								AttachedFile atfile = helper.getAttachedFile(i);
								//カメラ画像のバイナリデータ
								byte[] data = atfile.get_byte_array();
								res.setContentType(atfile.get_mime_type());
								//サーブレットでバイナリを扱うストリーム
								ServletOutputStream os = res.getOutputStream();
								//バッファリングすることで処理を軽くする
								BufferedOutputStream bs = new BufferedOutputStream(os);
								//カメラ画像がnullじゃない場合は画像をアップロード
								if(data!= null){
								bs.write(data);
								bs.close();
								//画像を保存する
								csm.set_camera_picture(robotid, data);
								}else{
									//画像が遅れている場合はdata=nullの場合がある。その場合画面がちらつくので過去の画像を
									//アップロードしてちらつき防止に使う。
									if(csm.get_camera_picture(robotid) != null){
									bs.write(csm.get_camera_picture(robotid));
									}else{
										//それでもない場合は0を送る(何も映らない)
										bs.write(0);
									}
									bs.close();
								}
							}
						}
						else{
							//画像の転送が遅延している場合
							ServletOutputStream os = res.getOutputStream();
							BufferedOutputStream bs = new BufferedOutputStream(os);
								if(csm.get_camera_picture(robotid) != null){
								bs.write(csm.get_camera_picture(robotid));
								}
							bs.close();
						}
					} catch (InterruptedException e) {
						// TODO 自動生成された catch ブロック
						//e.printStackTrace();
					}
				}
				//配信停止
				else if(display_mode != true){
					distribution_id = Long.valueOf(csm.getDistributionId(robotid));
					//multimedia_profileを取得
					mp = csm.getMultimediaProfile(robotid);
					try {
						//画像配信を停止
						mp.stop_distribute_camera_image(conv_id, distribution_id);
						//callbackクラスを削除
						csm.releaseCallback(robotid);
						//スレッド解除
						imgCallback.semaphore.release();
					} catch (RSiException e) {
						// TODO 自動生成された catch ブロック
						e.printStackTrace();
					}
				}
				//Ret_valueを破棄
				imgCallback.ret_value = null;
				imgCallback.semaphore.release();
		//workertoolが存在しない場合
		}
		//workertoolが無くなる場合があるので（切断）この条件も作る
		}else if(workertool == null){
			CameraCallback imgCallback = csm.getCallback(robotid);
			if(imgCallback == null){
				//何もしない
			}
			else if(imgCallback != null){
				//配信中でworkertoolにエラーが発生した恐れがあるため，再起動させる．
				//workertoolからはconv_idが取得できないためRobotWorkerManagerから取得する．
				csm.releaseCallback(robotid);
			}
		}
	}
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {
		//doPostに送ることで処理をひとまとめにする。
		doPost(req,res);
	}
}
